import base64

from cryptography.exceptions import UnsupportedAlgorithm
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.serialization import Encoding, PublicFormat, load_pem_public_key
from rest_framework import serializers

from arkindex.ponos.utils import get_process_from_task_auth


class PublicKeyField(serializers.CharField):
    """
    An EC public key, serialized in PEM format
    """

    default_error_messages = {
        "invalid_pem": "Incorrect PEM data",
        "unsupported_algorithm": "Key algorithm is not supported",
        "not_ec": "Key is not an EC public key",
    }

    def to_internal_value(self, data) -> ec.EllipticCurvePublicKey:
        data = super().to_internal_value(data)
        try:
            key = load_pem_public_key(
                data.encode("utf-8"),
                backend=default_backend(),
            )
        except ValueError:
            self.fail("invalid_pem")
        except UnsupportedAlgorithm:
            self.fail("unsupported_algorithm")

        if not isinstance(key, ec.EllipticCurvePublicKey):
            self.fail("not_ec")

        return key

    def to_representation(self, key: ec.EllipticCurvePublicKey) -> str:
        return key.public_bytes(
            Encoding.PEM,
            PublicFormat.SubjectPublicKeyInfo,
        ).decode("utf-8")


class Base64Field(serializers.CharField):
    """
    A base64-encoded bytestring.
    """

    def to_internal_value(self, data) -> bytes:
        return base64.b64decode(super().to_internal_value(data))

    def to_representation(self, obj: bytes) -> str:
        return base64.b64encode(obj)


class CurrentProcessDefault:
    """
    Use the process of the currently authenticated task as a default value.
    If Ponos task authentication is not in use, returns None.
    """
    requires_context = True

    def __call__(self, serializer_field):
        return get_process_from_task_auth(serializer_field.context["request"])
